// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package openim

import (
	"context"
	"fmt"
	"github.com/tidwall/gjson"

	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	"property-thirdparty/thirdparty"

	"time"

	"git.getensh.com/common/gopkgs/cache"

	"encoding/json"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

const OpenImTokenKey = "openim-token"

func openImToken() (string, error) {
	str, _ := cache.Redis().Get(OpenImTokenKey)
	if str != "" {
		return str, nil
	}

	pdata := map[string]interface{}{
		"secret":      parser.Conf.ThirdParty.OpenIm.Secret,
		"platform":    1,
		"userID":      parser.Conf.ThirdParty.OpenIm.AdminId,
		"operationID": fmt.Sprintf("%d%s", time.Now().Unix(), parser.Conf.ThirdParty.OpenIm.AdminId),
	}
	pbytes, _ := json.Marshal(pdata)

	hreq := thirdparty.HttpRequestWithHead{
		Url:    parser.Conf.ThirdParty.OpenIm.AdminTokenUrl,
		Head:   map[string]string{"Content-Type": "application/json"},
		Body:   pbytes,
		Method: "POST",
	}

	rbytes, err := hreq.Request()
	if err != nil {
		return "", err
	}
	code := gjson.GetBytes(rbytes, "errCode").Int()
	if code != 0 {
		msg := gjson.GetBytes(rbytes, "errMsg").String()
		logger.Error("thirdparty",
			zap.String("call", "openImToken"),
			zap.String("error", msg))
		return "", errors.VendorError
	}
	token := gjson.GetBytes(rbytes, "data").Get("token").String()
	if token == "" {
		logger.Error("thirdparty",
			zap.String("call", "openImToken"),
			zap.String("error", string(rbytes)))
		return "", errors.VendorError
	}
	_, _ = cache.Redis().SetEx(OpenImTokenKey, 60, token)

	return token, nil

}

func isOpenImRegistered(userId string) bool {

	token, err := openImToken()
	if err != nil {
		return false
	}

	pdata := map[string]interface{}{
		"checkUserIDList": []string{userId},
		"operationID":     fmt.Sprintf("%d%s", time.Now().Unix(), parser.Conf.ThirdParty.OpenIm.AdminId),
	}
	pbytes, _ := json.Marshal(pdata)

	hreq := thirdparty.HttpRequestWithHead{
		Url:    parser.Conf.ThirdParty.OpenIm.AccountCheckUrl,
		Head:   map[string]string{"Content-Type": "application/json", "token": token},
		Body:   pbytes,
		Method: "POST",
	}

	rbytes, err := hreq.Request()
	if err != nil {
		return false
	}
	code := gjson.GetBytes(rbytes, "errCode").Int()
	if code != 0 {
		msg := gjson.GetBytes(rbytes, "errMsg").String()
		logger.Error("thirdparty",
			zap.String("call", "isOpenImRegister"),
			zap.String("error", msg))
		return false
	}
	array := gjson.GetBytes(rbytes, "data").Array()
	for _, v := range array {
		if v.Get("userID").String() != userId {
			continue
		}
		if v.Get("accountStatus").String() == "registered" {
			return true
		}
	}

	return false
}

func OpenImRegister(ctx context.Context, req *pb_v1.OpenImRegisterRequest) (reply *pb_v1.OpenImRegisterReply, err error) {
	reply = &pb_v1.OpenImRegisterReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.UserId == "" || req.Platform == 0 || req.NickName == "" {
		return nil, errors.ParamsError
	}
	if isOpenImRegistered(req.UserId) {
		return reply, nil
	}

	pdata := map[string]interface{}{
		"secret":      parser.Conf.ThirdParty.OpenIm.Secret,
		"platform":    req.Platform,
		"userID":      req.UserId,
		"nickname":    req.NickName,
		"faceURL":     req.FaceUrl,
		"gender":      req.Gender,
		"operationID": fmt.Sprintf("%d%s", time.Now().Unix(), req.UserId),
	}
	pbytes, _ := json.Marshal(pdata)

	hreq := thirdparty.HttpRequestWithHead{
		Url:    parser.Conf.ThirdParty.OpenIm.RegisterUrl,
		Head:   map[string]string{"Content-Type": "application/json"},
		Body:   pbytes,
		Method: "POST",
	}

	rbytes, err := hreq.Request()
	if err != nil {
		return nil, err
	}
	code := gjson.GetBytes(rbytes, "errCode").Int()
	if code != 0 {
		msg := gjson.GetBytes(rbytes, "errMsg").String()
		logger.Error("thirdparty",
			zap.String("call", "OpenImRegister"),
			zap.String("error", msg))
		return reply, errors.VendorError
	}

	return reply, nil

}
