// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package openim

import (
	"context"
	"fmt"
	"github.com/tidwall/gjson"
	"strconv"
	"strings"

	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	"property-thirdparty/thirdparty"

	"time"

	"git.getensh.com/common/gopkgs/cache"

	"encoding/json"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

const OpenImTokenByUserKey = "openim-token-by-user"

func openImTokenByUser(platForm int32, openimId string) (string, int64, error) {
	key := fmt.Sprintf("%s-%s-%d", OpenImTokenByUserKey, openimId, platForm)
	str, _ := cache.Redis().Get(key)
	if str != "" {
		array := strings.Split(str, "----")
		if len(array) == 2 {
			timestamp, _ := strconv.ParseInt(array[1], 10, 64)
			return array[0], timestamp, nil
		}

	}

	pdata := map[string]interface{}{
		"secret":      parser.Conf.ThirdParty.OpenIm.Secret,
		"platform":    platForm,
		"userID":      openimId,
		"operationID": fmt.Sprintf("%d%s", time.Now().Unix(), parser.Conf.ThirdParty.OpenIm.AdminId),
	}
	pbytes, _ := json.Marshal(pdata)

	hreq := thirdparty.HttpRequestWithHead{
		Url:    parser.Conf.ThirdParty.OpenIm.AdminTokenUrl,
		Head:   map[string]string{"Content-Type": "application/json"},
		Body:   pbytes,
		Method: "POST",
	}

	rbytes, err := hreq.Request()
	if err != nil {
		return "", 0, err
	}
	code := gjson.GetBytes(rbytes, "errCode").Int()
	if code != 0 {
		msg := gjson.GetBytes(rbytes, "errMsg").String()
		logger.Error("thirdparty",
			zap.String("call", "openImToken"),
			zap.String("error", msg))
		return "", 0, errors.VendorError
	}
	token := gjson.GetBytes(rbytes, "data").Get("token").String()
	if token == "" {
		logger.Error("thirdparty",
			zap.String("call", "openImToken"),
			zap.String("error", string(rbytes)))
		return "", 0, errors.VendorError
	}
	now := time.Now().Unix()
	//expire := 24 * 3600
	expire := gjson.GetBytes(rbytes, "data").Get("expiredTime").Int()
	if expire > now+24*3600 || expire <= now {
		expire = now + 24*3600
	}
	
	_, _ = cache.Redis().SetEx(key, int64(expire-now), token)

	return token, int64(expire), nil

}

func OpenImToken(ctx context.Context, req *pb_v1.OpenImTokenRequest) (reply *pb_v1.OpenImTokenReply, err error) {
	reply = &pb_v1.OpenImTokenReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.OpenimId == "" || req.Platform == 0 {
		return nil, errors.ParamsError
	}

	reply.Token, reply.ExpireAt, err = openImTokenByUser(req.Platform, req.OpenimId)
	if err != nil {
		return nil, err
	}

	return reply, nil

}
