// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package openim

import (
	"context"
	"fmt"
	"github.com/tidwall/gjson"

	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	"property-thirdparty/thirdparty"

	"time"

	"encoding/json"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func OpenImUserInfo(ctx context.Context, req *pb_v1.OpenImUserInfoRequest) (reply *pb_v1.OpenImUserInfoReply, err error) {
	reply = &pb_v1.OpenImUserInfoReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if len(req.OpenimIds) == 0 || req.PlatformId == 0 {
		return nil, errors.ParamsError
	}
	token, _, err := openImTokenByUser(req.PlatformId, req.OpenimIds[0])
	if err != nil {
		return nil, err
	}
	pdata := map[string]interface{}{
		"operationID": fmt.Sprintf("%d%s", time.Now().Unix(), req.OpenimIds[0]),
		"userIDList":  req.OpenimIds,
	}

	pbytes, _ := json.Marshal(pdata)

	hreq := thirdparty.HttpRequestWithHead{
		Url:    parser.Conf.ThirdParty.OpenIm.UserInfoUrl,
		Head:   map[string]string{"Content-Type": "application/json", "token": token},
		Body:   pbytes,
		Method: "POST",
	}

	rbytes, err := hreq.Request()
	if err != nil {
		return nil, err
	}
	code := gjson.GetBytes(rbytes, "errCode").Int()
	if code != 0 {
		msg := gjson.GetBytes(rbytes, "errMsg").String()
		logger.Error("thirdparty",
			zap.String("call", "OpenImUserInfo"),
			zap.String("error", msg))
		return reply, errors.VendorError
	}
	array := gjson.GetBytes(rbytes, "data").Array()
	if len(array) == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.List = make([]*pb_v1.OpenImUserData, len(array))
	for i, _ := range array {
		openimId := array[i].Get("userID").String()
		reply.List[i] = &pb_v1.OpenImUserData{
			FaceUrl:  array[i].Get("faceURL").String(),
			NickName: array[i].Get("nickname").String(),
			Gender:   int32(array[i].Get("gender").Int()),
			OpenimId: openimId,
		}
	}

	return reply, nil

}
