// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package tencent_msg

import (
	"context"
	"fmt"
	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	"property-thirdparty/thirdparty"
	"property-thirdparty/utils"
	"sync"

	"encoding/json"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

const (
	TemplateMsgUrgType = 1
)

var TemplateMsgTypeM = map[int]string{}
var msgMux sync.Mutex

func initMsgType() {
	msgMux.Lock()
	defer msgMux.Unlock()
	if len(TemplateMsgTypeM) == 0 {
		TemplateMsgTypeM[TemplateMsgUrgType] = parser.Conf.ThirdParty.TencentMsg.UrgeTemplateId
	}
}

// 腾讯云短信消息发送
func TencentMsgSend(ctx context.Context, req *pb_v1.TencentMsgSendRequest) (reply *pb_v1.TencentMsgSendReply, err error) {
	reply = &pb_v1.TencentMsgSendReply{}
	initMsgType()
	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	// params verify

	if req.Phone != "" && !utils.VerifyMobileFormat(req.Phone) {
		return reply, status.Error(10003, "参数错误,手机号错误")
	}
	if len(req.Phones) > 0 {
		for _, v := range req.Phones {
			if !utils.VerifyMobileFormat(v) {
				return reply, status.Error(10003, "参数错误,手机号错误")
			}
		}
	}

	if req.Phone != "" {
		err = thirdparty.TencentMsg(req.Phone, req.Datas, TemplateMsgTypeM[int(req.MsgType)])
		if err != nil {
			logger.Error("func",
				zap.String("call", "ThirdParty.TencentMsg"),
				zap.String("params", req.Phone),
				zap.String("error", err.Error()))
			return reply, errors.MsgSendFailed
		}
	}
	if len(req.Phones) > 0 {
		start, end := 0, 0
		for {
			start = end
			if start >= len(req.Phones) {
				break
			}
			end = start + 200
			if end > len(req.Phones) {
				end = len(req.Phones)
			}

			err = thirdparty.TencentMsgs(req.Phones[start:end], req.Datas, TemplateMsgTypeM[int(req.MsgType)])
			if err != nil {
				logger.Error("func",
					zap.String("call", "ThirdParty.TencentMsgs"),
					zap.String("params", req.Phone),
					zap.String("error", err.Error()))
				return reply, errors.MsgSendFailed
			}
		}

	}

	return
}
