package wxapplet

import (
	"bytes"
	"context"
	gerrors "errors"
	"fmt"
	"git.getensh.com/common/gopkgs/cache"
	"git.getensh.com/common/gopkgs/logger"
	"github.com/go-pay/gopay/wechat"
	"github.com/tidwall/gjson"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"io/ioutil"
	"net/http"
	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
)

type Mini struct {
	Appid    string `json:"appid"`
	PagePath string `json:"pagepath"`
}
type WxPublicMsgParam struct {
	Touser      string                 `json:"touser"`
	TemplateId  string                 `json:"template_id"`
	Data        map[string]interface{} `json:"data"`
	Miniprogram Mini                   `json:"miniprogram"`
}

const PUBLICACCESSTOKENKEY = "wx_public_access_token"

func GetPublicAccessTokenFromWx() (string, int, error) {
	accessToken, err := wechat.GetAppletAccessToken(parser.Conf.ThirdParty.Wx.PublicAppId, parser.Conf.ThirdParty.Wx.PublicAppSecret)
	if err != nil {
		return "", 0, err
	}
	if accessToken.AccessToken == "" {
		return "", 0, gerrors.New(fmt.Sprintf("%v %v", accessToken.Errcode, accessToken.Errmsg))
	}
	return accessToken.AccessToken, accessToken.ExpiresIn, nil
}
func GetPublicAccessToken() (string, error) {
	accessToken, _ := cache.Redis().Get(PUBLICACCESSTOKENKEY)
	if accessToken != "" {
		return accessToken, nil
	}
	accessToken, expire, err := GetPublicAccessTokenFromWx()
	if err != nil {
		return "", err
	}
	PutPublicAccessToken(accessToken, expire)
	return accessToken, nil
}

func PutPublicAccessToken(token string, expire int) {
	if token == "" {
		return
	}
	if expire < 20 {
		return
	}
	cache.Redis().SetEx(PUBLICACCESSTOKENKEY, int64(expire-10), token)
}

const WxPublicMsgSendUrl = "https://api.weixin.qq.com/cgi-bin/message/template/send"

func WxPublicMsgSend(ctx context.Context, req *pb_v1.WxPublicMsgSendRequest) (reply *pb_v1.WxPublicMsgSendReply, err error) {
	reply = &pb_v1.WxPublicMsgSendReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	// 取access token
	token, err := GetPublicAccessToken()
	if err != nil {
		logger.Error("func",
			zap.String("call", "GetPublicAccessToken"),
			zap.String("error", err.Error()))
		return nil, errors.VendorError
	}

	// 组装参数
	p := WxPublicMsgParam{
		Touser:     req.PublicOpenId,
		TemplateId: req.TemplateId,
		//Miniprogram: Mini{Appid: parser.Conf.ThirdParty.Wx.AppletAppId, PagePath: "page/home"},
	}
	if parser.Conf.ThirdParty.Wx.AppletPagepath != "" {
		p.Miniprogram = Mini{Appid: parser.Conf.ThirdParty.Wx.AppletAppId, PagePath: parser.Conf.ThirdParty.Wx.AppletPagepath}
	}
	p.Data = map[string]interface{}{}
	for _, v := range req.Content {
		p.Data[v.Key] = map[string]interface{}{"value": v.Value, "color": "#173177"}

	}
	bytesData, _ := json.Marshal(p)
	fmt.Printf("%s\n", bytesData)

	// 推送消息
	reader := bytes.NewReader(bytesData)
	url := fmt.Sprintf("%s?access_token=%s", WxPublicMsgSendUrl, token)
	request, err := http.NewRequest("POST", url, reader)
	if err != nil {
		logger.Error("func",
			zap.String("call", "http.NewRequest"),
			zap.String("param", string(bytesData)),
			zap.String("error", err.Error()))
		return nil, errors.VendorError
	}

	request.Header.Set("Content-Type", "application/json")

	client := http.Client{}
	resp, err := client.Do(request)
	if err != nil {
		logger.Error("func",
			zap.String("call", "http.client.Do"),
			zap.String("param", string(bytesData)),
			zap.String("error", err.Error()))
		return nil, errors.VendorError
	}
	defer resp.Body.Close()

	// 处理消息结果
	respBytes, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		logger.Error("func",
			zap.String("call", "ioutil.ReadAll"),
			zap.String("error", err.Error()))
		return nil, errors.VendorError
	}

	errmsg := gjson.GetBytes(respBytes, "errmsg").String()
	errcode := gjson.GetBytes(respBytes, "errcode").Int()

	if errcode == 0 {
		return reply, nil
	}

	logger.Error("func",
		zap.String("call", "WxPublicMsgSend"),
		zap.String("param", string(bytesData)),
		zap.String("error", fmt.Sprintf("%d %s", errcode, errmsg)))

	return nil, errors.VendorError
}

func TestMsg() {
	mreq := pb_v1.WxPublicMsgSendRequest{}
	mreq.TemplateId = "U2uoP7LhrRPft6Rl5UroMu_J_ivaZTGRLcU1RdwPGSQ"
	mreq.PublicOpenId = "o4o5D6khn-6n19rSAiEz9CJyM0vo"
	mreq.Content = []*pb_v1.WxPublicMsgKeyValue{}
	item := &pb_v1.WxPublicMsgKeyValue{}
	item.Key = "keyword1"
	item.Value = "test"
	mreq.Content = append(mreq.Content, item)
	item = &pb_v1.WxPublicMsgKeyValue{}
	item.Key = "keyword2"
	item.Value = "test"
	mreq.Content = append(mreq.Content, item)
	item = &pb_v1.WxPublicMsgKeyValue{}
	item.Key = "first"
	item.Value = "testxx"
	mreq.Content = append(mreq.Content, item)
	item = &pb_v1.WxPublicMsgKeyValue{}
	item.Key = "remark"
	item.Value = "testyy"
	mreq.Content = append(mreq.Content, item)
	_, err := WxPublicMsgSend(context.Background(), &mreq)
	fmt.Printf("%v\n", err)
}
