package wxmerchant

import (
	"context"
	"crypto/rand"
	"crypto/rsa"
	"crypto/sha1"
	"encoding/base64"
	"encoding/json"
	"errors"
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"github.com/tidwall/gjson"
	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/services/certificates"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"io/ioutil"
	"log"
	"net/http"
	"net/url"
	"os"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
)

var publicKey *rsa.PublicKey
var serino string

func GetPlatformCert() (string, *rsa.PublicKey, error) {
	if publicKey != nil && serino != "" {
		return serino, publicKey, nil
	}
	mchID := parser.Conf.ThirdParty.Wx.MerchantMchID
	mchCertificateSerialNumber := parser.Conf.ThirdParty.Wx.MerchantMchCertificateSerialNumber
	mchAPIv3Key := parser.Conf.ThirdParty.Wx.MerchantMchAPIv3Key

	// 使用 utils 提供的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
	mchPrivateKey, err := utils.LoadPrivateKey(parser.Conf.ThirdParty.Wx.MerchantMchPrivateKey)
	if err != nil {
		log.Print("load merchant private key error")
		return "", nil, errors.New("xxxx")
	}

	ctx := context.Background()
	// 使用商户私钥等初始化 client，并使它具有自动定时获取微信支付平台证书的能力
	opts := []core.ClientOption{
		option.WithWechatPayAutoAuthCipher(mchID, mchCertificateSerialNumber, mchPrivateKey, mchAPIv3Key),
	}
	client, err := core.NewClient(ctx, opts...)
	if err != nil {
		log.Printf("new wechat pay client err:%s", err)
		return "", nil, errors.New("xxxx")
	}

	svc := certificates.CertificatesApiService{Client: client}
	resp, result, err := svc.DownloadCertificates(context.Background())
	if err != nil {
		fmt.Printf("aaaaaaaa:%v\n", err)
		return "", nil, errors.New("xxxx")
	}
	log.Printf("status=%d resp=%s", result.Response.StatusCode, resp)

	fmt.Printf("证书个数：%d\n", len(resp.Data))

	certPlain, err := utils.DecryptAES256GCM(mchAPIv3Key, *resp.Data[0].EncryptCertificate.AssociatedData, *resp.Data[0].EncryptCertificate.Nonce, *resp.Data[0].EncryptCertificate.Ciphertext)
	if err != nil {
		return "", nil, errors.New("xxxx")
	}
	cert, err := utils.LoadCertificate(certPlain)
	if err != nil {
		fmt.Printf("load x error:%v\n", err)
		return "", nil, errors.New("xxxx")
	}
	rsaPublicKey, ok := cert.PublicKey.(*rsa.PublicKey)
	if !ok {
		fmt.Printf("public key failed\n")
		return "", nil, errors.New("xxxx")
	}
	fmt.Printf("get plat success\n")
	serino = *resp.Data[0].SerialNo
	publicKey = rsaPublicKey
	return serino, publicKey, nil
}

func RsaEncrypt(src string) string {

	secretMessage := []byte(src)
	rng := rand.Reader

	_, rsaPublicKey, _ := GetPlatformCert()
	cipherdata, err := rsa.EncryptOAEP(sha1.New(), rng, rsaPublicKey, secretMessage, nil)
	if err != nil {
		fmt.Fprintf(os.Stderr, "Error from encryption: %s\n", err)
		panic("111")
		return ""
	}

	ciphertext := base64.StdEncoding.EncodeToString(cipherdata)

	fmt.Printf("encrypte:%s\n", ciphertext)
	return ciphertext
}

func ApplySubmch(req *pb_v1.WxMerchantApplyRequest, reply *pb_v1.WxMerchantApplyReply) string {
	mchID := parser.Conf.ThirdParty.Wx.MerchantMchID
	mchCertificateSerialNumber := parser.Conf.ThirdParty.Wx.MerchantMchCertificateSerialNumber
	mchAPIv3Key := parser.Conf.ThirdParty.Wx.MerchantMchAPIv3Key

	// 使用 utils 提供的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
	mchPrivateKey, err := utils.LoadPrivateKey(parser.Conf.ThirdParty.Wx.MerchantMchPrivateKey)
	if err != nil {
		return "私钥加载失败:" + err.Error()
	}

	ctx := context.Background()
	// 使用商户私钥等初始化 client，并使它具有自动定时获取微信支付平台证书的能力
	opts := []core.ClientOption{
		option.WithWechatPayAutoAuthCipher(mchID, mchCertificateSerialNumber, mchPrivateKey, mchAPIv3Key),
	}
	client, err := core.NewClient(ctx, opts...)
	if err != nil {
		return "core.NewCient failed:" + err.Error()
	}

	headers := http.Header{}
	platNo, _, _ := GetPlatformCert()
	headers.Set("Wechatpay-Serial", platNo)
	headers.Set("Accept", "application/json")
	headers.Set("Content-Type", "application/json")

	m := map[string]interface{}{
		"business_code":     req.BusinessCode,
		"contact_info":      getContactInfo(req),
		"subject_info":      getSubjectInfo(req),
		"business_info":     getBusinessInfo(req),
		"settlement_info":   getSettlementInfo(),
		"bank_account_info": getBankAccountInfo(req),
	}
	r, err := client.Request(context.Background(), "POST", "https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/", headers, url.Values{}, &m, "application/json")
	if err != nil {
		return "wx client Request failed:" + err.Error()
	}

	defer r.Response.Body.Close()
	contents, err := ioutil.ReadAll(r.Response.Body)
	if err != nil {
		return "read body faield:" + err.Error()
	}
	ret := gjson.GetBytes(contents, "applyment_id").String()
	if ret == "" {
		return fmt.Sprintf(fmt.Sprintf("未获取到申请号:%s", contents))
	}

	reply.ApplymentId = ret

	return ""
}

var identityTypeM = map[string]string{
	"IDENTIFICATION_TYPE_IDCARD":            "中国大陆居民-身份证",
	"IDENTIFICATION_TYPE_OVERSEA_PASSPORT":  "其他国家或地区居民-护照",
	"IDENTIFICATION_TYPE_HONGKONG_PASSPORT": "中国香港居民-来往内地通行证",
	"IDENTIFICATION_TYPE_MACAO_PASSPORT":    "中国澳门居民-来往内地通行证",
	"IDENTIFICATION_TYPE_TAIWAN_PASSPORT":   "中国台湾居民-来往大陆通行证",
}
var userTypeM = map[string]string{
	"LEGAL": "法人",
	"SUPER": "经办人",
}

func getContactInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {
	m := map[string]interface{}{
		"contact_type": req.ContactInfo.ContactType,
		"contact_name": RsaEncrypt(req.ContactInfo.ContactName),
		//"contact_id_doc_type":"IDENTIFICATION_TYPE_IDCARD",
		//"contact_id_number":RsaEncrypt("510922199301122334"),
		//"contact_id_doc_copy":"YmfSmW1IX3yH9xERqVCdzCTXZnE8ZNGjf8B7fDZsuJMUsRxjkAfvvExzabbS8Vi3-oPlM68e6y0EVPUE7nuf-7-z8JFplQ30rvg3OB66K2M",
		//"contact_id_doc_copy_back":"YmfSmW1IX3yH9xERqVCdzOnLv6GWdI-S0oat3tphrxYZuDAiL05pmegpL_WK5_pkDUvINx92nPHYy6bgG_N-Iw5ki1QX4E59WUSzngXBeBU",
		//"contact_period_begin":"2016-05-19",
		//"contact_period_end":"2026-05-19",
		"mobile_phone":  RsaEncrypt(req.ContactInfo.MobilePhone),
		"contact_email": RsaEncrypt(req.ContactInfo.ContactEmail),
	}
	if req.ContactInfo.ContactType != "LEGAL" {
		m["contact_id_doc_type"] = req.ContactInfo.ContactIdDocType
		m["contact_id_number"] = RsaEncrypt(req.ContactInfo.ContactIdNumber)
		m["contact_id_doc_copy"] = req.ContactInfo.ContactIdDocCopy
		m["contact_id_doc_copy_back"] = req.ContactInfo.ContactIdDocCopyBack
		m["contact_period_begin"] = req.ContactInfo.ContactPeriodBegin
		m["contact_period_end"] = req.ContactInfo.ContactPeriodEnd
	}
	return m
}

func getSubjectInfoBusinessLicenseInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {
	m := map[string]interface{}{
		"license_copy":    req.SubjectInfo.BusinessLicenseInfo.LicenseCopy,
		"license_number":  req.SubjectInfo.BusinessLicenseInfo.LicenseNumber,
		"merchant_name":   req.SubjectInfo.BusinessLicenseInfo.MerchantName,
		"legal_person":    req.SubjectInfo.BusinessLicenseInfo.LegalPerson,
		"license_address": req.SubjectInfo.BusinessLicenseInfo.LicenseAddress,
		"period_begin":    req.SubjectInfo.BusinessLicenseInfo.PeriodBegin,
		"period_end":      req.SubjectInfo.BusinessLicenseInfo.PeriodEnd,
	}
	return m
}

func getSubjectInfoIdentityInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {

	m := map[string]interface{}{
		//"id_doc_type":"IDENTIFICATION_TYPE_IDCARD",
		"owner": req.SubjectInfo.IdentityInfo.Owner,
	}

	if (req.SubjectInfo.SubjectType == "SUBJECT_TYPE_GOVERNMENT" ||
		req.SubjectInfo.SubjectType == "SUBJECT_TYPE_INSTITUTIONS") && req.SubjectInfo.IdentityInfo.IdHolderType == "SUPER" {
		m["id_holder_type"] = req.SubjectInfo.IdentityInfo.IdHolderType
	}
	if req.SubjectInfo.IdentityInfo.IdHolderType == "SUPER" {
		m["authorize_letter_copy"] = req.SubjectInfo.IdentityInfo.AuthorizeLetterCopy
	}
	if req.SubjectInfo.IdentityInfo.IdHolderType == "" || req.SubjectInfo.IdentityInfo.IdHolderType == "LEGAL" {
		m["id_doc_type"] = req.SubjectInfo.IdentityInfo.IdDocType
		if req.SubjectInfo.IdentityInfo.IdDocType == "IDENTIFICATION_TYPE_IDCARD" {
			m["id_card_info"] = map[string]interface{}{
				"id_card_copy":      req.SubjectInfo.IdentityInfo.IdCardInfo.IdCardCopy,
				"id_card_national":  req.SubjectInfo.IdentityInfo.IdCardInfo.IdCardNational,
				"id_card_name":      RsaEncrypt(req.SubjectInfo.IdentityInfo.IdCardInfo.IdCardName),
				"id_card_number":    RsaEncrypt(req.SubjectInfo.IdentityInfo.IdCardInfo.IdCardNumber),
				"id_card_address":   RsaEncrypt(req.SubjectInfo.IdentityInfo.IdCardInfo.IdCardAddress),
				"card_period_begin": req.SubjectInfo.IdentityInfo.IdCardInfo.CardPeriodBegin,
				"card_period_end":   req.SubjectInfo.IdentityInfo.IdCardInfo.CardPeriodEnd,
			}
		} else {
			m["id_doc_info"] = map[string]interface{}{
				"id_doc_copy":      req.SubjectInfo.IdentityInfo.IdDocInfo.IdDocCopy,
				"id_doc_copy_back": req.SubjectInfo.IdentityInfo.IdDocInfo.IdDocCopyBack,
				"id_doc_name":      RsaEncrypt(req.SubjectInfo.IdentityInfo.IdDocInfo.IdDocName),
				"id_doc_number":    RsaEncrypt(req.SubjectInfo.IdentityInfo.IdDocInfo.IdDocNumber),
				"id_doc_address":   RsaEncrypt(req.SubjectInfo.IdentityInfo.IdDocInfo.IdDocAddress),
				"doc_period_begin": req.SubjectInfo.IdentityInfo.IdDocInfo.DocPeriodBegin,
				"doc_period_end":   req.SubjectInfo.IdentityInfo.IdDocInfo.DocPeriodEnd,
			}
		}
	}

	return m
}

//SUBJECT_TYPE_GOVERNMENT,SUBJECT_TYPE_INSTITUTIONS,SUBJECT_TYPE_OTHERS,SUBJECT_TYPE_ENTERPRISE

func getSubjectInfoCertificateInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {
	m := map[string]interface{}{
		"cert_copy":       req.SubjectInfo.CertInfo.CertCopy,
		"cert_type":       req.SubjectInfo.CertInfo.CertType,
		"cert_number":     req.SubjectInfo.CertInfo.CertNumber,
		"merchant_name":   req.SubjectInfo.CertInfo.MerchantName,
		"company_address": req.SubjectInfo.CertInfo.CompanyAddress,
		"legal_person":    req.SubjectInfo.CertInfo.LegalPerson,
		"period_begin":    req.SubjectInfo.CertInfo.PeriodBegin,
		"period_end":      req.SubjectInfo.CertInfo.PeriodEnd,
	}
	return m
}

func getSubjectInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {
	m := map[string]interface{}{
		"subject_type":        req.SubjectInfo.SubjectType,
		"finance_institution": false,
		//"business_license_info":getSubjectInfoBusinessLicenseInfo(req),
		"identity_info": getSubjectInfoIdentityInfo(req),
	}
	if req.SubjectInfo.SubjectType == "SUBJECT_TYPE_ENTERPRISE" {
		m["business_license_info"] = getSubjectInfoBusinessLicenseInfo(req)
	} else {
		m["certificate_info"] = getSubjectInfoCertificateInfo(req)
	}
	if len(req.SubjectInfo.UboInfos) > 0 {
		array := make([]map[string]interface{}, len(req.SubjectInfo.UboInfos))
		for i, v := range req.SubjectInfo.UboInfos {
			array[i] = map[string]interface{}{
				"ubo_id_doc_type":      v.UboIdDocType,
				"ubo_id_doc_copy":      v.UboIdDocCopy,
				"ubo_id_doc_copy_back": v.UboIdDocCopyBack,
				"ubo_id_doc_name":      v.UboIdDocName,
				"ubo_id_doc_number":    v.UboIdDocNumber,
				"ubo_id_doc_address":   v.UboIdDocAddress,
				"ubo_period_begin":     v.UboPeriodBegin,
				"ubo_period_end":       v.UboPeriodEnd,
			}
		}
		m["ubo_info_list"] = array
	}
	return m
}

func getBusinessInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {
	m := map[string]interface{}{
		"merchant_shortname": req.BusinessInfo.MerchantShortname,
		"service_phone":      req.BusinessInfo.ServicePhone,
		"sales_info": map[string]interface{}{
			"sales_scenes_type": []string{"SALES_SCENES_MINI_PROGRAM"},
			"mini_program_info": map[string]interface{}{
				//todo
				"mini_program_sub_appid": parser.Conf.ThirdParty.Wx.AppletAppId,
			},
		},
	}
	return m
}

func getSettlementInfo() map[string]interface{} {
	m := map[string]interface{}{
		"settlement_id":      "716",
		"qualification_type": "居民生活服务",
	}
	return m
}

func getBankAccountInfo(req *pb_v1.WxMerchantApplyRequest) map[string]interface{} {
	m := map[string]interface{}{
		"bank_account_type": req.BankAccountInfo.BankAccountType,
		"account_name":      RsaEncrypt(req.BankAccountInfo.AccountName),
		"account_bank":      req.BankAccountInfo.AccountBank,
		"bank_address_code": req.BankAccountInfo.BankAddressCode,
		"bank_name":         req.BankAccountInfo.BankName,
		"account_number":    RsaEncrypt(req.BankAccountInfo.AccountNumber),
	}
	return m
}

func WxMerchantApply(ctx context.Context, req *pb_v1.WxMerchantApplyRequest) (reply *pb_v1.WxMerchantApplyReply, err error) {
	reply = &pb_v1.WxMerchantApplyReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	errMsg := ApplySubmch(req, reply)
	if errMsg != "" {
		logger.Error("func",
			zap.String("call", "ApplySubmch"),
			zap.String("error", errMsg))
		return reply, status.Error(10002, errMsg)
	}
	return reply, nil
}
